/*global $ jQuery*/
function Panel(settings) {
    // The title that would like the panel to have it if fails
    this.title = settings.title;
    // The element you would like to put the panel inside
    this.element = settings.element;
    // The url to query to get the panel
    this.url = settings.url;
    // What you would like to occur if the we are able to reach the url.
    this.success = settings.success;
    // What you would like to occur if we exceed numAttempts
    this.error = settings.error;
    // The number of attempts before calling settings.error.
    this.numAttempts= settings.numAttempts;
    // The number of consecutive successes that indicated a success.
    this.consecutiveSuccesses = settings.consecutiveSuccesses || 0;
    // How long to wait before each ping of the server.
    this.waitPeriod = settings.waitPeriod || 3000;
    // This function will be called on each attempt
    this.onAttempt = settings.onAttempt || function () {}; 
};

// get gets the remote panel from Panel.url
Panel.prototype.get = function (){
    // Calling this so that we don't need to reset numAttempts
    this.queryRemotePanel(this.numAttempts, this.consecutiveSuccesses);

};

// queryRemotePanel is helper function the does the actual query 
Panel.prototype.queryRemotePanel = function (numAttempts, consecutiveSuccesses) {
    var panel = this;
    jQuery.ajax({
        url: panel.url,
        method: "GET",
        success: function(data, xhr, status){
            if (consecutiveSuccesses <= 0 ) {
                panel.success(data, xhr, status);
            } else {
                consecutiveSuccesses -= 1;
                panel.retryRequest(numAttempts, consecutiveSuccesses);
            }
            
        },
        error: function (jqXTR, status, error) {
            if (numAttempts <= 0) {
                panel.error();
            } else {
                panel.retryRequest(numAttempts, panel.consecutiveSuccesses);
            }
        },
        timeout: 5000
    });


};



Panel.prototype.retryRequest = function(numAttempts, consecutiveSuccesses) {
    var panel = this;
    
    window.setTimeout(function() {
        panel.onAttempt(panel.numAttempts - numAttempts, consecutiveSuccesses);
        numAttempts -= 1;
        panel.queryRemotePanel(numAttempts, consecutiveSuccesses);
    }, panel.waitPeriod);
    
    
};

Panel.prototype.displayErrorPanel = function() {
    var panel = this;
    var errorPanel = $("#error-panel").clone();
    var onRetry =  function() {
        displayLoader(panel.element, panel.title);
        panel.get();
    };
    displayErrorPanel({
        title: panel.title, 
        container: panel.element, 
        onRetry: onRetry, 
        example: errorPanel
    });
    
};



Panel.prototype.showPanel = function(remotePanel) {
    console.log("We ARE TO SHOW PANEL");
    var panel = this;
    $(panel.element).html(remotePanel);
};


function displayLoader(id, name) {
    console.log(id);
    console.log(name);
    var loader = $("<div class='large-loader'\>");
    var loadingName = $("<span/>").addClass("loader-inner-text").append(name);
    $(id).html('');
    $(id).append(loader);
    $(id).append(loadingName);
}

function displayErrorPanel(settings) {
    settings.example = settings.example || $("#error-panel");
    settings.titleContainer = settings.titleContainer || ".content-panel-title";
    settings.onRetry = settings.onRetry || function() {};
    
    var errorPanel = $(settings.example).clone();
    $(errorPanel).attr("id", settings.title+"-error-panel");
    $(errorPanel).find(settings.titleContainer).append(settings.title);
    $(errorPanel).find(".retry-panel-load").on("click", function() {
        settings.onRetry();
    });
    $(settings.container).html(errorPanel);
    
}





